// -*- C++ -*-
/*!
 * @file  ARMarkerDetectTest.h
 * @brief Test module for AR Marker Detection
 * @date  $Date$
 *
 * @author Tsuyoshi Suenaga, NAIST
 * tsuyo-s@is.naist.jp
 *
 * GNU General Public License
 *
 * $Id$
 */

#ifndef ARMARKERDETECTTEST_H
#define ARMARKERDETECTTEST_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/idl/ExtendedDataTypesSkel.h>
#include <rtm/idl/InterfaceDataTypesSkel.h>
//#include <rtm/idl/MyDataType.h>

// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">
#include "MyDataTypeStub.h"

// </rtc-template>

using namespace RTC;
using namespace MyDataType;

/*!
 * @class ARMarkerDetectTest
 * @brief Test module for AR Marker Detection
 *
 * ARToolkitを用いたマーカ検出RTCの動作テスト用プログラムで
 * す。サービスポートから検出するマーカIDを指定し、マーカ
 * の位置姿勢を取得、表示します。
 *
 */
class ARMarkerDetectTest
  : public RTC::DataFlowComponentBase
{
 public:
  /*!
   * @brief constructor
   * @param manager Maneger Object
   */
  ARMarkerDetectTest(RTC::Manager* manager);

  /*!
   * @brief destructor
   */
  ~ARMarkerDetectTest();

  // <rtc-template block="public_attribute">
  
  // </rtc-template>

  // <rtc-template block="public_operation">
  
  // </rtc-template>

  /***
   *
   * The initialize action (on CREATED->ALIVE transition)
   * formaer rtc_init_entry() 
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onInitialize();

  /***
   *
   * The finalize action (on ALIVE->END transition)
   * formaer rtc_exiting_entry()
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onFinalize();

  /***
   *
   * The startup action when ExecutionContext startup
   * former rtc_starting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  /***
   *
   * The shutdown action when ExecutionContext stop
   * former rtc_stopping_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  /***
   *
   * The activated action (Active state entry action)
   * former rtc_active_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  /***
   *
   * The deactivated action (Active state exit action)
   * former rtc_active_exit()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  /***
   *
   * The execution action that is invoked periodically
   * former rtc_active_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  /***
   *
   * The aborting action when main logic error occurred.
   * former rtc_aborting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  /***
   *
   * The error action in ERROR state
   * former rtc_error_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  /***
   *
   * The reset action that is invoked resetting
   * This is same but different the former rtc_init_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  /***
   *
   * The state update action that is invoked after onExecute() action
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  /***
   *
   * The action that is invoked when execution context's rate is changed
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // <rtc-template block="protected_attribute">
  
  // </rtc-template>

  // <rtc-template block="protected_operation">
  
  // </rtc-template>

  // Configuration variable declaration
  // <rtc-template block="config_declare">
  /*!
   * 位置姿勢を取得したいマーカのID．サービスポートRecogni
   * tionServiceよりIDが送信され、入力ポートより位置姿勢を取得す
   * る。
   * - Name: 検出対象ID detect_marker_id
   * - DefaultValue: 0
   */
  long int m_detect_marker_id;
  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  TimedObjectPoseMatSeq m_markerPoseAll;
  /*!
   * 検出されたマーカの位置姿勢をID番号と共に全て取得する
   * 。
   */
  InPort<TimedObjectPoseMatSeq> m_markerPoseAllIn;
  TimedDoubleSeq m_markerPose;
  /*!
   * サービスポートRecognitionServicceにより指定したマーカの位
   * 置姿勢を取得する。出力対象のマーカIDは、始めはConfiguratio
   * nパラーメータで指定されたIDであり、サービスポートのRecog
   * nitionServiceにより任意のIDも指定可能である。
   * - Type: double
   * - Number: 12
   * - Semantics: 以下のような行列があった場合、
   *              r1 r2 r3 t1
   *              r4 r5 r6 t2
   *              r7 r8 r9 t3
   *               0  0  0  1
   *              r1, r2, r3, t1, r4, r5, r6, t2, r7, r8, r9, t3
   *              として出力する．
   */
  InPort<TimedDoubleSeq> m_markerPoseIn;
  
  // </rtc-template>


  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  /*!
   */
  RTC::CorbaPort m_ObjectRecognitionPortPort;
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  /*!
   */
  RTC::CorbaConsumer<RecognitionService> m_RecognitionService;
  
  // </rtc-template>

 private:
  // <rtc-template block="private_attribute">
  
  // </rtc-template>

  // <rtc-template block="private_operation">
  
  // </rtc-template>
  bool *port_connect_check;

};


extern "C"
{
  DLL_EXPORT void ARMarkerDetectTestInit(RTC::Manager* manager);
};

#endif // ARMARKERDETECTTEST_H
